/* xgraph.c */
/* A Turbo Pascal 6.0 GRAPH.TPU like interface to X11. */
/* (c) Heiko Purnhagen <purnhage@tnt.uni-hannover.de> */
/* $Id: xgraph.c,v 1.3 1998/12/30 12:50:08 purnhage Exp $ */
/* HP 980908 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include "xgraph.h"


static char *color_values[NCOLORS] = { 
  "#000000",
  "#000080",
  "#008000",
  "#008080",
  "#800000",
  "#800080",
  "#808000",
  "#c0c0c0",
  "#808080",
  "#0000ff",
  "#00ff00",
  "#00ffff",
  "#ff0000",
  "#ff00ff",
  "#ffff00",
  "#ffffff"
};

static int xgraphOK = 0;
static int xgraphColor = NCOLORS-1;

static Display *displ;
static int screen;
static Window wind;
static GC gc_draw;
static Colormap cmap;
static unsigned long colors[NCOLORS];
static int borderwidth,xorig,yorig,xsize,ysize;
static int widthmm,heightmm;
static int depth;
static XSetWindowAttributes w_attr;
static unsigned long w_mask;
static XGCValues gc_values;
static unsigned long val_mask;
static XColor xcolor;

void InitGraph (int *GraphDriver, int *GraphMode, char *PathToDriver)
{
  int i;

  if (xgraphOK)
    return;

  /* open X window */
  displ = XOpenDisplay(getenv("DISPLAY"));
  if (displ==NULL) {
    fprintf(stderr,"xtest: No connection to display %s\n",
	    XDisplayName(NULL));
    exit(-1);
  }
  screen = DefaultScreen(displ);
  depth = DefaultDepth(displ,screen);
  w_attr.border_pixel = WhitePixel(displ,screen);
  w_attr.background_pixel = BlackPixel(displ,screen);
  w_attr.override_redirect = True;
  w_mask = (CWBackPixel | CWBorderPixel | CWOverrideRedirect);
  borderwidth = 0;
  xorig = 0;
  yorig = 0;
  xsize = DisplayWidth(displ,screen);
  ysize = DisplayHeight(displ,screen);
  widthmm = DisplayWidthMM(displ,screen);
  heightmm = DisplayHeightMM(displ,screen);
  wind = XCreateWindow(displ,RootWindow(displ,screen),
		       xorig,yorig,xsize,ysize,borderwidth,depth,
		       InputOutput,CopyFromParent,w_mask,&w_attr);
  XMapWindow(displ,wind);
  gc_values.foreground = BlackPixel(displ,screen);
  val_mask = GCForeground;
  gc_draw = XCreateGC(displ,wind,val_mask,&gc_values);

  /* XSetLineAttributes(displ,gc_draw,1,LineOnOffDash,CapNotLast,JoinMiter); */
  cmap = DefaultColormap(displ,screen);
  for(i=0;i<NCOLORS;i++) {
    XParseColor(displ,cmap,color_values[i],&xcolor);
    XAllocColor(displ,cmap,&xcolor);
    colors[i] = xcolor.pixel;
  }

  xgraphOK = 1;
}

int GraphResult (void)
{
  return xgraphOK ? 0 : 1;
}

void RestoreCrtMode (void)
{
}

void CloseGraph (void)
{
  if (!xgraphOK)
    return;

  /* close x window */
  XCloseDisplay(displ);
}


int GetMaxColor (void)
{
  return NCOLORS-1;
}

int GetMaxX (void)
{
  return xsize-1;
}

int GetMaxY (void)
{
  return ysize-1;
}


void SetColor(int Color)
{
  xgraphColor = Color%NCOLORS;
}

void Line (int x1, int y1, int x2, int y2)
{
  XSetForeground(displ,gc_draw,colors[xgraphColor%NCOLORS]);
  XDrawLine(displ,wind,gc_draw,x1,y1,x2,y2);
}

void PutPixel (int x, int y, int Pixel)
{
  XSetForeground(displ,gc_draw,colors[Pixel%NCOLORS]);
  XDrawPoint(displ,wind,gc_draw,x,y);
}


/* end of xgraph.c */
