/***

xtest -- X11 info and CRT test picture

$Id: xtest.c,v 1.6 2000/11/14 23:59:19 purnhage Exp $

(c) Heiko Purnhagen 1998
e-mail: purnhage@tnt.uni-hannover.de
WWW:    http://www.tnt.uni-hannover.de/~purnhage/

All usual disclaimers apply!

This program may be copied according to the GNU General Public Licence.
See file COPYING for further details.

HP 970914 980803 980806 981220 20001112 20001113 20001114

usage: xtest h
       ... to show online help ;-)

***/


#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/X.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include <math.h>


#define PROGVER "xtest -- X11 info and CRT test picture   V1.0 30-dec-98"
#define CVSID "$Id: xtest.c,v 1.6 2000/11/14 23:59:19 purnhage Exp $"
#ifndef DISTVER		/* set in Makefile */
#define DISTVER "<unknown>"
#endif

#ifndef M_PI
#define M_PI        3.14159265358979323846      /* pi */
#endif

#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) > (y) ? (x) : (y))
#define limit(x,a,b) (min((b),max((a),(x))))

#define NCOLORS 8

char *color_values[NCOLORS] = { 
  "#000000","#ff0000","#00ff00","#0000ff",
  "#00ffff","#ff00ff","#ffff00","#ffffff"
};

Display *displ;
int screen;
Window wind;
GC gc_draw,gc_fill;
Colormap cmap;
unsigned long colors[NCOLORS];
int borderwidth,xorig,yorig,xsize,ysize;
int widthmm,heightmm;
int depth;
XSetWindowAttributes w_attr;
unsigned long w_mask;
XGCValues gc_values;
unsigned long val_mask;
XColor xcolor;

int i,j,k;
int mode;


void circle (int x0, int y0, float r, int steps)
{
  int j;
  float p1,p2;
  int x1,y1,x2,y2;

  for (j=0; j<=steps; j++) {
    p1 = j*2*M_PI/steps;
    p2 = (j+1)*2*M_PI/steps;
    x1 = x0+(int)(r*sin(p1));
    y1 = y0+(int)(r*cos(p1));
    x2 = x0+(int)(r*sin(p2));
    y2 = y0+(int)(r*cos(p2));
    XDrawLine(displ,wind,gc_draw,x1,y1,x2,y2);
  }
}

int main (int argc, char *argv[])
{
  if (argc < 2 || *argv[1] < '1' || *argv[1] > '5') {
    printf(PROGVER "   " DISTVER "\n"
	   CVSID "\n"
	   "\n"
	   "xtest [mode]\n"
	   "mode: 1 = show X11 resolution/size\n"
	   "      2 = grid & concentric circles   (<ret> to change colour)\n"
           "                                      (q <ret> to quit)\n"
	   "      3 = squares & circles           (<ret> to change colour)\n"
           "                                      (q <ret> to quit)\n"
	   "      4 = colours ...                 (^C to quit)\n"
	   "      5 = chess (grey)                (<ret> to invert)\n"
           "                                      (q <ret> to quit)\n"
	   "      6 = lissajous 1:2.1             (^C to quit)\n"
	   "      h = show this online help ...\n");
    return 0;
  }
  mode = *argv[1] - '0';

  /* open X window */
  displ = XOpenDisplay(getenv("DISPLAY"));
  if (displ==NULL) {
    fprintf(stderr,"xtest: No connection to display %s\n",
	    XDisplayName(NULL));
    exit(-1);
  }
  screen = DefaultScreen(displ);
  depth = DefaultDepth(displ,screen);
  w_attr.border_pixel = WhitePixel(displ,screen);
  w_attr.background_pixel = BlackPixel(displ,screen);
  w_attr.override_redirect = True;
  w_mask = (CWBackPixel | CWBorderPixel | CWOverrideRedirect);
  borderwidth = 0;
  xorig = 0;
  yorig = 0;
  xsize = DisplayWidth(displ,screen);
  ysize = DisplayHeight(displ,screen);
  widthmm = DisplayWidthMM(displ,screen);
  heightmm = DisplayHeightMM(displ,screen);
  wind = XCreateWindow(displ,RootWindow(displ,screen),
		       xorig,yorig,xsize,ysize,borderwidth,depth,
		       InputOutput,CopyFromParent,w_mask,&w_attr);
  XMapWindow(displ,wind);
  gc_values.foreground = BlackPixel(displ,screen);
  val_mask = GCForeground;
  gc_draw = XCreateGC(displ,wind,val_mask,&gc_values);

  /* XSetLineAttributes(displ,gc_draw,1,LineOnOffDash,CapNotLast,JoinMiter); */
  cmap = DefaultColormap(displ,screen);
  for(i=0;i<NCOLORS;i++) {
    XParseColor(displ,cmap,color_values[i],&xcolor);
    XAllocColor(displ,cmap,&xcolor);
    colors[i] = xcolor.pixel;
  }
  
  /* draw stuff */

  if (mode == 4) {
    for (i=0; i<NCOLORS; i++) {
      XSetForeground(displ,gc_draw,colors[i]);
      XDrawLine(displ,wind,gc_draw,0,i,xsize-1,ysize-1-i);
    }
    XFlush(displ);
    for (j=0; j<ysize; j++)
      for (i=0; i<xsize; i++) {
	XSetForeground(displ,gc_draw,colors[rand()%NCOLORS]);
	XDrawPoint(displ,wind,gc_draw,i,j);
      }
    XFlush(displ);
    getchar();
  }

  else if (mode == 5) {
    XSetForeground(displ,gc_draw,colors[7]);
    for (j=0; j<ysize; j++)
      for (i=j%2; i<xsize; i=i+2) {
	XDrawPoint(displ,wind,gc_draw,i,j);
      }
    XFlush(displ);
    getchar();
    k = 0;
    while (1) {
      XSetForeground(displ,gc_draw,colors[7]);
      XDrawLine(displ,wind,gc_draw,0,0,xsize-1,0);
      XDrawLine(displ,wind,gc_draw,0,ysize-1,xsize-1,ysize-1);
      XDrawLine(displ,wind,gc_draw,0,0,0,ysize-1);
      XDrawLine(displ,wind,gc_draw,xsize-1,0,xsize-1,ysize-1);
      XDrawLine(displ,wind,gc_draw,0,1,xsize-2,ysize-1);
      XDrawLine(displ,wind,gc_draw,0,0,xsize-1,ysize-1);
      XDrawLine(displ,wind,gc_draw,1,0,xsize-1,ysize-2);
      XDrawLine(displ,wind,gc_draw,0,ysize-2,xsize-2,0);
      XDrawLine(displ,wind,gc_draw,0,ysize-1,xsize-1,0);
      XDrawLine(displ,wind,gc_draw,1,ysize-1,xsize-1,1);
      XFlush(displ);
      if (getchar()=='q')
	break;
      for (j=0; j<ysize; j++)
	for (i=0; i<xsize; i++) {
	  XSetForeground(displ,gc_draw,colors[7*((i+j+k)%2)]);
	  XDrawPoint(displ,wind,gc_draw,i,j);
	}
      XFlush(displ);
      if (getchar()=='q')
	break;
      k = (k+1)%2;
    }
  }

  else if (mode == 2 || mode == 3) {
    float r;
    int x0,y0;
    
    k = 7;
    while (1) {
      XSetForeground(displ,gc_draw,colors[k]);
      
      XDrawLine(displ,wind,gc_draw,0,0,xsize-1,0);
      XDrawLine(displ,wind,gc_draw,0,ysize-1,xsize-1,ysize-1);
      XDrawLine(displ,wind,gc_draw,0,0,0,ysize-1);
      XDrawLine(displ,wind,gc_draw,xsize-1,0,xsize-1,ysize-1);
      
      XDrawLine(displ,wind,gc_draw,0,0,xsize-1,ysize-1);
      XDrawLine(displ,wind,gc_draw,0,ysize-1,xsize-1,0);
      
      if (mode == 2) {
	for (i=0; i<8; i++) {
	  j = (ysize/16)*i;
	  XDrawLine(displ,wind,gc_draw,0,j,xsize-1,j);
	  j = ysize-1-j;
	  XDrawLine(displ,wind,gc_draw,0,j,xsize-1,j);
	  j = ysize/2;
	  XDrawLine(displ,wind,gc_draw,0,j,xsize-1,j);
	  j = (xsize/16)*i;
	  XDrawLine(displ,wind,gc_draw,j,0,j,ysize-1);
	  j = xsize-1-j;
	  XDrawLine(displ,wind,gc_draw,j,0,j,ysize-1);
	  j = xsize/2;
	  XDrawLine(displ,wind,gc_draw,j,0,j,ysize-1);
	}
      }
      else {
	for (j=0; j<ysize; j+=16)
	  XDrawLine(displ,wind,gc_draw,0,j,xsize-1,j);
	for (j=0; j<xsize; j+=16)
	  XDrawLine(displ,wind,gc_draw,j,0,j,ysize-1);
      }

      if (mode == 2) {
	for (i=1; i<=4; i++) {
	  x0 = xsize/2; 
	  y0 = ysize/2;
	  r = min(xsize,ysize)/8.*i;
	  circle(x0,y0,r,256);
	}
      }
      else {
	r = 32;
	for (i=32; i+32<xsize/2; i+=64)
	  for (j=32; j+32<ysize/2; j+=64) {
	    circle(i,j,r,32);
	    circle(xsize-1-i,j,r,32);
	    circle(i,ysize-1-j,r,32);
	    circle(xsize-1-i,ysize-1-j,r,32);
	  }
      }

      XFlush(displ);
      if (getchar()=='q')
	break;
      k = (k+1)%8;
    }
  }

  else if (mode == 6) {
#define LEN 4000
#define SPD 100
    int x[LEN],y[LEN];
    int xx,yy,xxx,yyy;
    float px=0;
    float py=0;
    float pdx=2*M_PI/4000;
    float pdy=pdx*2.1;
    
    for (i=0; i<LEN; i++)
      x[i] = y[i] = 0;
    xx = yy = xxx = yyy = 0;
    i = 0;
    while (1) {
      XSetForeground(displ,gc_draw,colors[0]);
      XDrawPoint(displ,wind,gc_draw,x[i],y[i]);
#if 0
      xxx += (rand()%3)-1;
      yyy += (rand()%3)-1;
      xxx = limit(xxx,-SPD,SPD);
      yyy = limit(yyy,-SPD,SPD);
      xx += xxx;
      yy += yyy;
      xx = (xx+xsize*SPD)%(xsize*SPD);
      yy = (yy+ysize*SPD)%(ysize*SPD);
      xx = limit(xx,0,xsize*SPD-1);
      yy = limit(yy,0,ysize*SPD-1);
      x[i] = xx/SPD;
      y[i] = yy/SPD;
#endif
      x[i] = (int)((sin(px)*.5+.5)*(xsize-1));
      y[i] = (int)((sin(py)*.5+.5)*(ysize-1));
      px += pdx;
      if (px > 2*M_PI)
	px -= 2*M_PI;
      py += pdy;
      if (py > 2*M_PI)
	py -= 2*M_PI;
      XSetForeground(displ,gc_draw,colors[7]);
      XDrawPoint(displ,wind,gc_draw,x[i],y[i]);
      i = (i+1)%LEN;
    }
  }

  /* close x window */
  
  XCloseDisplay(displ);
  
  printf("xsize=%d ysize=%d depth=%d wid=%dmm height=%dmm\n",
	 xsize,ysize,depth,widthmm,heightmm);
  
  return 0;
}


